@@ -95,6 +95,7 @@
 #define kVLCSettingBackupMediaLibraryDefaultValue @NO
 #define kVLCSettingLastUsedSubtitlesSearchLanguage @"kVLCSettingLastUsedSubtitlesSearchLanguage"
 #define kVLCResetSettings @"kVLCResetSettings"
+#define kVLCSettingAlwaysPlayURLs @"kVLCSettingAlwaysPlayURLs"
 
 #define kVLCForceSMBV1 @"smb-force-v1"
 
@@ -155,6 +156,9 @@
 #define kVLCSortDefault @"SortDefault"
 #define kVLCSortDescendingDefault @"SortDescendingDefault"
 #define kVLCHasLaunchedBefore @"hasLaunchedBefore"
+#define kVLCHasNaggedThisMonth @"kVLCHasNaggedThisMonth"
+#define kVLCNumberOfLaunches @"kVLCNumberOfLaunches"
+#define kVLCHasActiveSubscription @"kVLCHasActiveSubscription"
 
 #define kVLCTabBarIndex @"TabBarIndex"
 
@@ -162,6 +166,8 @@
 
 #define kVLCEqualizerSnapBands @"EqualizerSnapBands"
 
+#define kVLCDonationAnonymousCustomerID @"kVLCDonationAnonymousCustomerID"
+
 /* LEGACY KEYS, DO NOT USE IN NEW CODE */
 #define kVLCFTPServer @"ftp-server"
 #define kVLCFTPLogin @"ftp-login"
