@@ -151,17 +151,21 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
     BOOL enforceFullscreen = [[defaults objectForKey:kVLCSettingVideoFullscreenPlayback] boolValue];
 
-    if (self.playbackController.fullscreenSessionRequested && enforceFullscreen) {
-        [self showFullscreenPlayback];
-        return;
-    }
-
     VLCMedia *currentMedia = _playbackController.currentlyPlayingMedia;
     VLCMLMedia *media = [VLCMLMedia mediaForPlayingMedia:currentMedia];
 
+    _playbackController.fullscreenSessionRequested = [media type] != VLCMLMediaTypeAudio;
+
+    if (self.playbackController.fullscreenSessionRequested && enforceFullscreen) {
+        [self setDisplayMode:VLCPlayerDisplayControllerDisplayModeFullscreen];
+    } else if ([media type] == VLCMLMediaTypeAudio && _playbackController.numberOfVideoTracks > 0) {
+        [self setDisplayMode:VLCPlayerDisplayControllerDisplayModeFullscreen];
+    }
+
     switch (self.displayMode) {
         case VLCPlayerDisplayControllerDisplayModeFullscreen:
-            if (media.type == VLCMLMediaTypeAudio) {
+            if ((media.type == VLCMLMediaTypeAudio || _playbackController.playAsAudio) &&
+                _playbackController.numberOfVideoTracks == 0) {
                 [self _presentAudioPlayerViewIfNeeded];
             } else {
                 [self _presentFullscreenPlaybackViewIfNeeded];
@@ -315,6 +319,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
     UIViewController<VLCPlaybackServiceDelegate> *movieViewController = self.movieViewController;
     UINavigationController *navCon = [[VLCPlaybackNavigationController alloc] initWithRootViewController:movieViewController];
     navCon.modalPresentationStyle = UIModalPresentationOverFullScreen;
+    navCon.modalPresentationCapturesStatusBarAppearance = YES;
     [movieViewController prepareForMediaPlayback:self.playbackController];
 
     UIWindow *window = [[UIApplication sharedApplication] keyWindow];
@@ -328,6 +333,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
     UIViewController<VLCPlaybackServiceDelegate> *audioPlayerViewController = self.audioPlayerViewController;
     UINavigationController *navCon = [[VLCPlaybackNavigationController alloc] initWithRootViewController:audioPlayerViewController];
     navCon.modalPresentationStyle = UIModalPresentationOverFullScreen;
+    navCon.modalPresentationCapturesStatusBarAppearance = YES;
     [audioPlayerViewController prepareForMediaPlayback:self.playbackController];
 
     UIWindow *window = [[UIApplication sharedApplication] keyWindow];
@@ -369,6 +375,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 
     void (^completionBlock)(BOOL) = nil;
     if (needsShow) {
+        // Init the mini player view if needed
         if (!miniPlaybackView) {
             UIViewController *rootViewController = UIApplication.sharedApplication.keyWindow.rootViewController;
 
@@ -394,15 +401,27 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
                 _trailingConstraint = [miniPlaybackView.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor];
             }
 
+            NSLayoutConstraint* heightConstraint = [miniPlaybackView.heightAnchor constraintEqualToConstant:((UIView<VLCPlaybackServiceDelegate, VLCMiniPlayer>*)self.miniPlaybackView).contentHeight];
+            heightConstraint.priority = UILayoutPriorityDefaultHigh;
+
             [NSLayoutConstraint activateConstraints:
              @[_bottomConstraint,
-               [miniPlaybackView.heightAnchor constraintEqualToConstant:((UIView<VLCPlaybackServiceDelegate, VLCMiniPlayer>*)self.miniPlaybackView).contentHeight],
+               heightConstraint,
                _leadingConstraint,
                _trailingConstraint,
                ]];
             [((VLCAudioMiniPlayer*)_miniPlaybackView) setupQueueViewControllerWith:_queueViewController];
             [self.view layoutIfNeeded];
         }
+
+        // Properly set the shuffle and repeat mode
+        NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
+        if ([userDefaults boolForKey:kVLCPlayerShouldRememberState]) {
+            _playbackController.shuffleMode = [userDefaults boolForKey:kVLCPlayerIsShuffleEnabled];
+            NSInteger repeatMode = [userDefaults integerForKey:kVLCPlayerIsRepeatEnabled];
+            _playbackController.repeatMode = repeatMode;
+        }
+
         [self addPlayqueueToMiniPlayer];
         miniPlaybackView.visible = YES;
         [[NSNotificationCenter defaultCenter]
@@ -445,7 +464,10 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
                          }
                          completion:completionBlock];
     }
-    [playbackController savePlaybackState];
+
+    if (needsHide) {
+        [playbackController savePlaybackState];
+    }
 #endif
 }
 
@@ -526,7 +548,6 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 - (void)videoPlayerViewControllerDidMinimize:(VLCVideoPlayerViewController *)videoPlayerViewController
 {
     [self closeFullscreenPlayback];
-    [self addPlayqueueToMiniPlayer];
 }
 
 - (BOOL)videoPlayerViewControllerShouldBeDisplayed:(VLCVideoPlayerViewController *)videoPlayerViewController
@@ -534,12 +555,19 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
     return self.displayMode == VLCPlayerDisplayControllerDisplayModeFullscreen;
 }
 
+- (void)videoPlayerViewControllerShouldSwitchPlayer:(VLCVideoPlayerViewController *)videoPlayerViewController
+{
+    [_movieViewController dismissViewControllerAnimated:[self shouldAnimate] completion:^{
+        [self setDisplayMode:VLCPlayerDisplayControllerDisplayModeFullscreen];
+        [self _presentFullscreenPlaybackViewIfNeeded];
+    }];
+}
+
 #pragma mark - AudioPlayerViewControllerDelegate
 
 - (void)audioPlayerViewControllerDidMinimize:(VLCAudioPlayerViewController *)audioPlayerViewController
 {
     [self closeAudioPlayer];
-    [self addPlayqueueToMiniPlayer];
 }
 
 - (void)audioPlayerViewControllerDidClose:(VLCAudioPlayerViewController *)audioPlayerViewController
@@ -554,6 +582,14 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
     return self.displayMode == VLCPlayerDisplayControllerDisplayModeFullscreen;
 }
 
+- (void)audioPlayerViewControllerShouldSwitchPlayer:(VLCAudioPlayerViewController *)audioPlayerViewController
+{
+    [_audioPlayerViewController dismissViewControllerAnimated:[self shouldAnimate] completion:^{
+        [self setDisplayMode:VLCPlayerDisplayControllerDisplayModeFullscreen];
+        [self _presentFullscreenPlaybackViewIfNeeded];
+    }];
+}
+
 #pragma mark - KeyCommands
 
 - (void)handlePlayPauseKeyInput
