@@ -16,6 +16,7 @@ protocol AudioPlayerViewControllerDelegate: AnyObject {
     func audioPlayerViewControllerDidMinimize(_ audioPlayerViewController: AudioPlayerViewController)
     func audioPlayerViewControllerDidClose(_ audioPlayerViewController: AudioPlayerViewController)
     func audioPlayerViewControllerShouldBeDisplayed(_ audioPlayerViewController: AudioPlayerViewController) -> Bool
+    func audioPlayerViewControllerShouldSwitchPlayer(_ audioPlayerViewController: AudioPlayerViewController)
 }
 
 @objc (VLCAudioPlayerViewController)
@@ -27,11 +28,15 @@ class AudioPlayerViewController: PlayerViewController {
     private var isQueueHidden: Bool = true
 
     override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
-        get { return UIInterfaceOrientationMask.portrait }
+        get { return UIInterfaceOrientationMask.allButUpsideDown }
+    }
+
+    override var preferredStatusBarStyle: UIStatusBarStyle {
+        return .lightContent
     }
 
     lazy var audioPlayerView: AudioPlayerView = {
-        let audioPlayerView: AudioPlayerView = Bundle.main.loadNibNamed("AudioPlayerView", owner: nil)?.first as! AudioPlayerView
+        let audioPlayerView = AudioPlayerView(frame: .zero)
         audioPlayerView.delegate = self
         return audioPlayerView
     }()
@@ -56,18 +61,20 @@ class AudioPlayerViewController: PlayerViewController {
 
     @objc override init(mediaLibraryService: MediaLibraryService, rendererDiscovererManager: VLCRendererDiscovererManager, playerController: PlayerController) {
         super.init(mediaLibraryService: mediaLibraryService, rendererDiscovererManager: rendererDiscovererManager, playerController: playerController)
+        NotificationCenter.default.addObserver(self, selector: #selector(playbackSpeedHasChanged(_:)), name: Notification.Name("ChangePlaybackSpeed"), object: nil)
 
         self.playerController.delegate = self
         mediaNavigationBar.addMoreOptionsButton(moreOptionsButton)
         audioPlayerView.setupNavigationBar(with: mediaNavigationBar)
-        audioPlayerView.setupThumbnailView()
+        audioPlayerView.updateThumbnailImageView()
+        audioPlayerView.setupPlaybackSpeed()
         audioPlayerView.setupBackgroundColor()
-        audioPlayerView.setupPlayerControls()
         mediaScrubProgressBar.updateBackgroundAlpha(with: 0.0)
         audioPlayerView.setupProgressView(with: mediaScrubProgressBar)
         audioPlayerView.setupExternalOutputView(with: externalOutputView)
         setupAudioPlayerViewConstraints()
         setupOptionsNavigationBar()
+        setupSliders()
         setupStatusLabel()
     }
 
@@ -80,20 +87,37 @@ class AudioPlayerViewController: PlayerViewController {
         navigationController?.navigationBar.isHidden = true
         playbackService.delegate = self
         playbackService.recoverPlaybackState()
-        seekForwardBy = UserDefaults.standard.integer(forKey: kVLCSettingPlaybackForwardSkipLength)
-        seekBackwardBy = UserDefaults.standard.integer(forKey: kVLCSettingPlaybackBackwardSkipLength)
-        audioPlayerView.setupThumbnailView()
+        audioPlayerView.updateThumbnailImageView()
         audioPlayerView.setupBackgroundColor()
+        audioPlayerView.setupPlaybackSpeed()
+        mediaScrubProgressBar.updateInterfacePosition()
         setupGestures()
+        playModeUpdated()
 
         if playbackService.isPlayingOnExternalScreen() {
             changeOutputView(to: externalOutputView.displayView)
         }
+
+        let orientation = getDeviceOrientation()
+        audioPlayerView.updateConstraints(for: orientation)
+        mediaScrubProgressBar.shouldHideScrubLabels = orientation.isLandscape ? true : false
+
+        let displayShortcutView: Bool = UserDefaults.standard.bool(forKey: kVLCPlayerShowPlaybackSpeedShortcut)
+        audioPlayerView.shouldDisplaySecondaryStackView(displayShortcutView)
     }
 
     override func viewWillDisappear(_ animated: Bool) {
         playerController.isInterfaceLocked = false
         queueViewController?.hide()
+        numberOfGestureSeek = 0
+        totalSeekDuration = 0
+        previousSeekState = .default
+    }
+
+    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
+        let orientation = getDeviceOrientation()
+        audioPlayerView.updateConstraints(for: orientation)
+        mediaScrubProgressBar.shouldHideScrubLabels = orientation.isLandscape ? true : false
     }
 
     // MARK: Public methods
@@ -101,6 +125,10 @@ class AudioPlayerViewController: PlayerViewController {
     override func minimizePlayer() {
         delegate?.audioPlayerViewControllerDidMinimize(self)
     }
+    
+    @objc func playbackSpeedHasChanged(_ notification: NSNotification) {
+        audioPlayerView.setupPlaybackSpeed()
+    }
 
     override func showPopup(_ popupView: PopupView, with contentView: UIView, accessoryViewsDelegate: PopupViewAccessoryViewsDelegate? = nil) {
         moreOptionsButton.isEnabled = false
@@ -142,6 +170,7 @@ class AudioPlayerViewController: PlayerViewController {
         audioPlayerView.thumbnailView.addGestureRecognizer(panRecognizer)
         audioPlayerView.thumbnailView.addGestureRecognizer(leftSwipeRecognizer)
         audioPlayerView.thumbnailView.addGestureRecognizer(rightSwipeRecognizer)
+        audioPlayerView.thumbnailView.addGestureRecognizer(doubleTapGestureRecognizer)
         audioPlayerView.addGestureRecognizer(playPauseRecognizer)
         audioPlayerView.addGestureRecognizer(pinchRecognizer)
 
@@ -167,6 +196,22 @@ class AudioPlayerViewController: PlayerViewController {
         audioPlayerView.thumbnailView.isHidden = true
     }
 
+    override func handleDoubleTapGesture(_ sender: UITapGestureRecognizer) {
+        let screenWidth: CGFloat = view.frame.size.width
+        let middleBoundary: CGFloat = screenWidth / 2.0
+
+        let tapPosition = sender.location(in: view)
+
+        // Reset number(set to -1/1) of seek when orientation has been changed.
+        if tapPosition.x < middleBoundary {
+            numberOfGestureSeek = previousSeekState == .forward ? -1 : numberOfGestureSeek - 1
+        } else {
+            numberOfGestureSeek = previousSeekState == .backward ? 1 : numberOfGestureSeek + 1
+        }
+
+        super.handleDoubleTapGesture(sender)
+    }
+
     // MARK: - Private methods
 
     private func setupAudioPlayerViewConstraints() {
@@ -187,7 +232,16 @@ class AudioPlayerViewController: PlayerViewController {
         view.addSubview(optionsNavigationBar)
         NSLayoutConstraint.activate([
             optionsNavigationBar.topAnchor.constraint(equalTo: audioPlayerView.navigationBarView.bottomAnchor, constant: padding),
-            optionsNavigationBar.trailingAnchor.constraint(equalTo: audioPlayerView.trailingAnchor, constant: -padding)
+            optionsNavigationBar.trailingAnchor.constraint(equalTo: audioPlayerView.layoutGuide.trailingAnchor, constant: -padding)
+        ])
+    }
+
+    private func setupSliders() {
+        audioPlayerView.setupSliders(with: brightnessControlView, and: volumeControlView)
+
+        NSLayoutConstraint.activate([
+            brightnessControlView.topAnchor.constraint(greaterThanOrEqualTo: optionsNavigationBar.bottomAnchor),
+            volumeControlView.topAnchor.constraint(greaterThanOrEqualTo: optionsNavigationBar.bottomAnchor)
         ])
     }
 
@@ -242,6 +296,30 @@ class AudioPlayerViewController: PlayerViewController {
 
         playerController.isInterfaceLocked = !enabled
     }
+
+    private func getDeviceOrientation() -> UIDeviceOrientation {
+        // Return the correct device orientation even if it is detected
+        // as flat.
+        let orientation = UIDevice.current.orientation
+
+        if orientation.isFlat {
+            let statusBarOrientation = UIApplication.shared.statusBarOrientation
+            switch statusBarOrientation {
+            case .portrait:
+                return .portrait
+            case .landscapeLeft:
+                return .landscapeLeft
+            case .landscapeRight:
+                return .landscapeRight
+            case .portraitUpsideDown:
+                return .portraitUpsideDown
+            default:
+                return .unknown
+            }
+        }
+
+        return orientation
+    }
 }
 
 // MARK: - AudioPlayerViewDelegate
@@ -255,9 +333,13 @@ extension AudioPlayerViewController: AudioPlayerViewDelegate {
 
         return image
     }
+    
+    func audioPlayerViewDelegateGetPlaybackSpeed(_ audioPlayerView: AudioPlayerView) -> Float {
+        return playbackService.playbackRate
+    }
 
-    func audioPlayerViewDelegateDidTapBackwardButton(_ audioPlayerView: AudioPlayerView) {
-        playbackService.jumpBackward(Int32(seekBackwardBy))
+    func audioPlayerViewDelegateDidTapShuffleButton(_ audioPlayerView: AudioPlayerView) {
+        updateShuffleState()
     }
 
     func audioPlayerViewDelegateDidTapPreviousButton(_ audioPlayerView: AudioPlayerView) {
@@ -273,8 +355,32 @@ extension AudioPlayerViewController: AudioPlayerViewDelegate {
         playbackService.next()
     }
 
-    func audioPlayerViewDelegateDidTapForwardButton(_ audioPlayerView: AudioPlayerView) {
-        playbackService.jumpForward(Int32(seekForwardBy))
+    func audioPlayerViewDelegateDidTapRepeatButton(_ audioPlayerView: AudioPlayerView) {
+        updateRepeatMode()
+    }
+
+    func audioPlayerViewDelegateDidTapPlaybackSpeedButton(_ audioPlayerView: AudioPlayerView) {
+        var currentSpeed = playbackService.playbackRate
+        let speedOffset: Float = 0.25
+
+        if currentSpeed + speedOffset > 2.0 {
+            currentSpeed = 1.0
+            mediaMoreOptionsActionSheetHideIcon(for: .playbackSpeed)
+        } else {
+            currentSpeed += speedOffset
+            mediaMoreOptionsActionSheetShowIcon(for: .playbackSpeed)
+        }
+
+        playbackService.playbackRate = currentSpeed
+        audioPlayerView.setupPlaybackSpeed()
+        NotificationCenter.default.post(name: Notification.Name("ChangePlaybackSpeed"), object: nil)
+    }
+
+    func audioPlayerViewDelegateDidLongPressPlaybackSpeedButton(_ audioPlayerView: AudioPlayerView) {
+        present(moreOptionsActionSheet, animated: false) {
+            [unowned self] in
+            self.moreOptionsActionSheet.addView(.playback)
+        }
     }
 
     func audioPlayerViewDelegateGetBrightnessSlider(_ audioPlayerView: AudioPlayerView) -> BrightnessControlView {
@@ -291,6 +397,7 @@ extension AudioPlayerViewController: AudioPlayerViewDelegate {
 extension AudioPlayerViewController {
     func prepare(forMediaPlayback playbackService: PlaybackService) {
         audioPlayerView.updatePlayButton(isPlaying: playbackService.isPlaying)
+        audioPlayerView.updateShuffleRepeatState(shuffleEnabled: playbackService.isShuffleMode, repeatMode: playbackService.repeatMode)
 
         let metadata = playbackService.metadata
         audioPlayerView.updateLabels(title: metadata.title, artist: metadata.artist, isQueueHidden: isQueueHidden)
@@ -315,7 +422,15 @@ extension AudioPlayerViewController {
         audioPlayerView.updatePlayButton(isPlaying: isPlaying)
 
         let image: UIImage? = isPlaying ? UIImage(named: "minimize") : UIImage(named: "close")
-        mediaNavigationBar.updateCloseButton(with: image)
+        let accessibilityLabel: String = isPlaying ? NSLocalizedString("MINIMIZE_BUTTON", comment: "") : NSLocalizedString("STOP_BUTTON", comment: "")
+        let accessibilityHint: String = isPlaying ? NSLocalizedString("MINIMIZE_HINT", comment: "") : NSLocalizedString("CLOSE_HINT", comment: "")
+        mediaNavigationBar.updateCloseButton(with: image, accessibility: (accessibilityLabel, accessibilityHint))
+
+        if currentState == .opening && playbackService.numberOfVideoTracks > 0 {
+            // This media contains video tracks and can be played with the Video Player.
+            delegate?.audioPlayerViewControllerShouldSwitchPlayer(self)
+            return
+        }
 
         if let queueCollectionView = queueViewController?.queueCollectionView {
             queueCollectionView.reloadData()
@@ -329,6 +444,12 @@ extension AudioPlayerViewController {
         if currentState == .buffering {
             mediaDuration = playbackService.mediaDuration
         }
+
+        if currentState == .opening || currentState == .stopped {
+            resetABRepeat()
+        }
+
+        moreOptionsActionSheet.currentMediaHasChapters = currentMediaHasChapters
     }
 
     func displayMetadata(for playbackService: PlaybackService, metadata: VLCMetaData) {
@@ -336,7 +457,7 @@ extension AudioPlayerViewController {
         updateNavigationBar(with: isQueueHidden ? nil : metadata.title)
 
         if metadata.artworkImage != audioPlayerView.thumbnailImageView.image {
-            audioPlayerView.setupThumbnailView()
+            audioPlayerView.updateThumbnailImageView()
             audioPlayerView.setupBackgroundColor()
 
             if let qvc = queueViewController, !isQueueHidden {
@@ -344,6 +465,10 @@ extension AudioPlayerViewController {
             }
         }
     }
+
+    func playModeUpdated() {
+        audioPlayerView.updateShuffleRepeatState(shuffleEnabled: playbackService.isShuffleMode, repeatMode: playbackService.repeatMode)
+    }
 }
 
 // MARK: - PlayerControllerDelegate
@@ -360,10 +485,6 @@ extension AudioPlayerViewController: PlayerControllerDelegate {
     func playerControllerApplicationBecameActive(_ playerController: PlayerController) {
         // TODO
     }
-
-    func playerControllerPlaybackDidStop(_ playerController: PlayerController) {
-        delegate?.audioPlayerViewControllerDidMinimize(self)
-    }
 }
 
 // MARK: - MediaNavigationBarDelegate
@@ -427,12 +548,40 @@ extension AudioPlayerViewController {
             view.addSubview(bookmarksView)
             NSLayoutConstraint.activate([
                 bookmarksView.centerXAnchor.constraint(equalTo: view.centerXAnchor),
-                bookmarksView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
-                bookmarksView.trailingAnchor.constraint(equalTo: view.trailingAnchor),
-                bookmarksView.topAnchor.constraint(equalTo: view.topAnchor, constant: 16),
-                bookmarksView.bottomAnchor.constraint(lessThanOrEqualTo: audioPlayerView.controlsStackView.topAnchor),
+                bookmarksView.leadingAnchor.constraint(equalTo: audioPlayerView.layoutGuide.leadingAnchor),
+                bookmarksView.trailingAnchor.constraint(equalTo: audioPlayerView.layoutGuide.trailingAnchor),
+                bookmarksView.topAnchor.constraint(equalTo: audioPlayerView.layoutGuide.topAnchor, constant: 16),
+                bookmarksView.bottomAnchor.constraint(equalTo: audioPlayerView.controlsStackView.topAnchor),
             ])
         }
+
+        audioPlayerView.shouldDisableControls(true)
+    }
+
+    override func mediaMoreOptionsActionSheetRemoveAddBookmarksView() {
+        super.mediaMoreOptionsActionSheetRemoveAddBookmarksView()
+
+        audioPlayerView.shouldDisableControls(false)
+    }
+
+    func mediaMoreOptionsActionSheetShowPlaybackSpeedShortcut(_ displayView: Bool) {
+        audioPlayerView.shouldDisplaySecondaryStackView(displayView)
+    }
+
+    override func mediaMoreOptionsActionSheetPresentABRepeatView(with abView: ABRepeatView) {
+        super.mediaMoreOptionsActionSheetPresentABRepeatView(with: abView)
+
+        guard let abRepeatView = abRepeatView else {
+            return
+        }
+
+        audioPlayerView.addSubview(abRepeatView)
+        audioPlayerView.bringSubviewToFront(abRepeatView)
+        abRepeatView.isUserInteractionEnabled = true
+        NSLayoutConstraint.activate([
+            abRepeatView.centerXAnchor.constraint(equalTo: audioPlayerView.layoutGuide.centerXAnchor),
+            abRepeatView.bottomAnchor.constraint(equalTo: mediaScrubProgressBar.topAnchor, constant: -10.0),
+        ])
     }
 }
 
