@@ -43,6 +43,7 @@ class QueueViewController: UIViewController {
     @IBOutlet weak var closeButton: UIButton!
 
     private var scrolledCellIndex: IndexPath = IndexPath()
+    private var grabbedCellIndex: IndexPath?
 
     private let cellHeight: CGFloat = 56
 
@@ -59,9 +60,10 @@ class QueueViewController: UIViewController {
             PlaybackService.sharedInstance()
         }
     }
+
     private var mediaList: VLCMediaList {
         get {
-            PlaybackService.sharedInstance().mediaList
+            playbackService.isShuffleMode ? PlaybackService.sharedInstance().shuffledList : PlaybackService.sharedInstance().mediaList
         }
     }
 
@@ -79,6 +81,8 @@ class QueueViewController: UIViewController {
     }
     var bottomConstraint: NSLayoutConstraint?
 
+    var heightConstraint: NSLayoutConstraint?
+
     private var darkOverlayView: UIView = UIView()
     private var darkOverlayViewConstraints: [NSLayoutConstraint] = []
 
@@ -99,14 +103,20 @@ class QueueViewController: UIViewController {
     override func viewDidLoad() {
         super.viewDidLoad()
         initViews()
-        NotificationCenter.default.addObserver(self,
-                                               selector: #selector(themeDidChange),
-                                               name: .VLCThemeDidChangeNotification,
-                                               object: nil)
-        NotificationCenter.default.addObserver(self,
-                                               selector: #selector(deviceOrientationDidChange),
-                                               name: UIDevice.orientationDidChangeNotification,
-                                               object: nil)
+
+        let defaultCenter: NotificationCenter = NotificationCenter.default
+        defaultCenter.addObserver(self,
+                                  selector: #selector(themeDidChange),
+                                  name: .VLCThemeDidChangeNotification,
+                                  object: nil)
+        defaultCenter.addObserver(self,
+                                  selector: #selector(deviceOrientationDidChange),
+                                  name: UIDevice.orientationDidChangeNotification,
+                                  object: nil)
+        defaultCenter.addObserver(self,
+                                  selector: #selector(reload),
+                                  name: Notification.Name(VLCPlaybackServiceShuffleModeUpdated),
+                                  object: nil)
     }
 
     override func viewWillDisappear(_ animated: Bool) {
@@ -136,7 +146,10 @@ class QueueViewController: UIViewController {
                 darkOverlayView.bottomAnchor.constraint(equalTo: parent.view.bottomAnchor)
             ]
 
-            let heightConstraint: NSLayoutConstraint?
+            if let heightConstraint = heightConstraint {
+                view.removeConstraint(heightConstraint)
+            }
+
             var miniPlayerView: AudioMiniPlayer? = nil
             if let parent = parent as? VLCPlayerDisplayController, let miniPlaybackView = parent.miniPlaybackView as? AudioMiniPlayer {
                 grabberView.isHidden = true
@@ -156,7 +169,7 @@ class QueueViewController: UIViewController {
                     bottomConstraint = view.bottomAnchor.constraint(equalTo: parent.view.bottomAnchor,
                                                                     constant: self.view.frame.height)
                 } else if let parent = parent as? AudioPlayerViewController {
-                    heightConstraint = view.heightAnchor.constraint(equalToConstant: parent.audioPlayerView.playqueueView.bounds.height)
+                    heightConstraint = nil
                     grabberView.isHidden = true
                     closeButton.isHidden = true
                     topConstraint = view.topAnchor.constraint(equalTo: parent.audioPlayerView.playqueueView.topAnchor)
@@ -413,9 +426,6 @@ private extension QueueViewController {
         if isSelected {
             textColor = PresentationTheme.current.colors.orangeUI
             tintColor = PresentationTheme.current.colors.orangeUI
-            cell.disableScrollView()
-        } else {
-            cell.enableScrollView()
         }
 
         cell.tintColor = tintColor
@@ -438,12 +448,31 @@ private extension QueueViewController {
                     break
             }
             queueCollectionView.beginInteractiveMovementForItem(at: selectedIndexPath)
+            grabbedCellIndex = selectedIndexPath
         case .changed:
             var location = gesture.location(in: gesture.view)
             location.x = queueCollectionView.frame.width / 2
             queueCollectionView.updateInteractiveMovementTargetPosition(location)
+            if let selectedIndexPath = queueCollectionView.indexPathForItem(at: gesture.location(in: queueCollectionView)) {
+                grabbedCellIndex = selectedIndexPath
+            }
         case .ended:
             queueCollectionView.endInteractiveMovement()
+            var indexPath: IndexPath? = nil
+
+            if let selectedIndexPath = queueCollectionView.indexPathForItem(at: gesture.location(in: queueCollectionView)) {
+                indexPath = selectedIndexPath
+            } else if let grabbedCellIndex = grabbedCellIndex {
+                indexPath = grabbedCellIndex
+            }
+
+            guard let index = indexPath,
+                  let cell = queueCollectionView.cellForItem(at: index) as? MediaCollectionViewCell else {
+                break
+            }
+
+            cell.animateCurrentlyPlayingState()
+            break
         default:
             queueCollectionView.cancelInteractiveMovement()
         }
@@ -486,17 +515,16 @@ extension QueueViewController: UICollectionViewDelegate, MediaCollectionViewCell
             assertionFailure("QueueViewController: didSelectItemAt: IndexPath out of range.")
             return
         }
-        let media = mediaList.media(at: UInt(indexPath.row))
-        let isPlaying = playbackService.currentlyPlayingMedia == media
-        if !isPlaying {
-            playbackService.playItem(at: UInt(indexPath.row))
-            guard let cell = collectionView.cellForItem(at: indexPath) as? MediaCollectionViewCell else {
-                assertionFailure("QueueViewController: didSelectItemAt: Cell not a MediaCollectionViewCell")
-                return
-            }
-            updateCollectionViewCellApparence(cell, isSelected: true)
-            reload()
+
+        playbackService.playItem(at: UInt(indexPath.row))
+
+        guard let cell = collectionView.cellForItem(at: indexPath) as? MediaCollectionViewCell else {
+            assertionFailure("QueueViewController: didSelectItemAt: Cell not a MediaCollectionViewCell")
+            return
         }
+
+        updateCollectionViewCellApparence(cell, isSelected: true)
+        reload()
     }
 
     func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
