@@ -146,16 +146,6 @@ class SettingsController: UITableViewController {
         UIApplication.shared.openURL(URL(string: "https://docs.videolan.me/vlc-user/ios/3.X/en/index.html")!)
     }
 
-    @objc private func showTipJar() {
-        if #available(iOS 10, *) {
-            ImpactFeedbackGenerator().selectionChanged()
-        }
-        let donationVC = VLCDonationViewController(nibName: "VLCDonationViewController", bundle: nil)
-        let donationNC = UINavigationController(rootViewController: donationVC)
-        donationNC.modalTransitionStyle = .flipHorizontal
-        present(donationNC, animated: true, completion: nil)
-    }
-
     @objc private func themeDidChange() {
         self.view.backgroundColor = PresentationTheme.current.colors.background
         setNavBarAppearance()
@@ -287,62 +277,10 @@ class SettingsController: UITableViewController {
         present(alert, animated: true)
     }
 
-    private func resetOption(for preferenceKey: String?) {
-        specifierManager.preferenceKey = preferenceKey
-        specifierManager.settingsBundle = settingsBundle
-        specifierManager.reset()
-    }
-
     private func resetOptions() {
-        GenericOptions.allCases.forEach {
-            if $0 == GenericOptions.automaticallyPlayNextItem {
-                let userDefaults = UserDefaults.standard
-                userDefaults.setValue(true, forKey: kVLCAutomaticallyPlayNextItem)
-                userDefaults.setValue(true, forKey: kVLCPlaylistPlayNextItem)
-            } else {
-                resetOption(for: $0.preferenceKey)
-            }
-        }
-
-        PrivacyOptions.allCases.forEach {
-            resetOption(for: $0.preferenceKey)
-        }
-
-        PlaybackControlOptions.allCases.forEach {
-            resetOption(for: $0.preferenceKey)
-        }
-
-        VideoOptions.allCases.forEach {
-            resetOption(for: $0.preferenceKey)
-        }
-
-        SubtitlesOptions.allCases.forEach {
-            resetOption(for: $0.preferenceKey)
-        }
-
-        AudioOptions.allCases.forEach {
-            resetOption(for: $0.preferenceKey)
-        }
-
-        CastingOptions.allCases.forEach {
-            resetOption(for: $0.preferenceKey)
-        }
-
-        MediaLibraryOptions.allCases.forEach {
-            if $0 != MediaLibraryOptions.forceVLCToRescanTheMediaLibrary {
-                resetOption(for: $0.preferenceKey)
-            }
-        }
-
-        NetworkOptions.allCases.forEach {
-            resetOption(for: $0.preferenceKey)
-        }
-
-        Lab.allCases.forEach {
-            if $0 != Lab.exportLibrary {
-                resetOption(for: $0.preferenceKey)
-            }
-        }
+        // note that [NSUserDefaults resetStandardUserDefaults] will NOT correctly reset to the defaults
+        let appDomain = Bundle.main.bundleIdentifier!
+        UserDefaults().removePersistentDomain(forName: appDomain)
     }
 }
 
@@ -516,8 +454,15 @@ extension SettingsController {
             playHaptics(sectionType: mainSection)
             showActionSheet(for: mainSection)
         case .donation:
-            showTipJar()
-            return
+            if #available(iOS 10, *) {
+                ImpactFeedbackGenerator().selectionChanged()
+            }
+            let donationVC = VLCDonationViewController(nibName: "VLCDonationViewController", bundle: nil)
+            let donationNC = UINavigationController(rootViewController: donationVC)
+            donationNC.modalPresentationStyle = .popover
+            donationNC.modalTransitionStyle = .flipHorizontal
+            donationNC.popoverPresentationController?.sourceView = tableView.cellForRow(at: indexPath)
+            present(donationNC, animated: true, completion: nil)
         case .generic:
             let genericSection = GenericOptions(rawValue: indexPath.row)
             playHaptics(sectionType: genericSection)
